/*
	Copyright (C) 2020-2021 Reiko Asakura. All Rights Reserved.

	Music Premium
*/

#include <libdbg.h>

#include "patch.h"

/* ARGSUSED */
int hook_export(
	const char *module, uint32_t libnid, uint32_t funcnid, const void *func,
	SceUID *hook_id, tai_hook_ref_t *hook_ref, const char *name)
{
	*hook_id = taiHookFunctionExportForKernel(KERNEL_PID, hook_ref, module, libnid, funcnid, func);
	if (*hook_id < 0) {
		SCE_DBG_LOG_ERROR("Failed to hook %s %08X", name, *hook_id);
	} else {
		SCE_DBG_LOG_INFO("Hooked %s %08X", name, *hook_id);
	}
	return *hook_id;
}

/* ARGSUSED */
int hook_abs(void *dest, const void *func, SceUID *hook_id, tai_hook_ref_t *hook_ref, const char *name) {
	*hook_id = taiHookFunctionAbs(KERNEL_PID, hook_ref, dest, func);
	if (*hook_id < 0) {
		SCE_DBG_LOG_ERROR("Failed to hook %s %08X", name, *hook_id);
	} else {
		SCE_DBG_LOG_INFO("Hooked %s %08X", name, *hook_id);
	}
	return *hook_id;
}

/* ARGSUSED */
int unhook(SceUID *hook_id, tai_hook_ref_t hook_ref, const char *name) {
	int ret = SCE_OK;
	if (*hook_id < 0) {
		SCE_DBG_LOG_WARNING("Skipped unhooking %s %08X", name, *hook_id);
	} else {
		ret = taiHookReleaseForKernel(*hook_id, hook_ref);
		if (ret == SCE_OK) {
			SCE_DBG_LOG_INFO("Unhooked %s %08X", name, *hook_id);
			*hook_id = -1;
		} else {
			SCE_DBG_LOG_ERROR("Failed to unhook %s %08X %08X", name, *hook_id, ret);
		}
	}
	return ret;
}

/* ARGSUSED */
int inject_abs(void *dest, const void *src, size_t size, SceUID *inject_id, const char *name) {
	*inject_id = taiInjectAbsForKernel(KERNEL_PID, dest, src, size);
	if (*inject_id < 0) {
		SCE_DBG_LOG_ERROR("Failed to inject %s %08X", name, *inject_id);
	} else {
		SCE_DBG_LOG_INFO("Injected %s %08X", name, *inject_id);
	}
	return *inject_id;
}

/* ARGSUSED */
int uninject(SceUID *inject_id, const char *name) {
	int ret = SCE_OK;
	if (*inject_id < 0) {
		SCE_DBG_LOG_WARNING("Skipped uninjecting %s %08X", name, *inject_id);
	} else {
		ret = taiInjectReleaseForKernel(*inject_id);
		if (ret == SCE_OK) {
			SCE_DBG_LOG_INFO("Uninjected %s %08X", name, *inject_id);
			*inject_id = -1;
		} else {
			SCE_DBG_LOG_ERROR("Failed to uninject %s %08X %08X", name, *inject_id, ret);
		}
	}
	return ret;
}
